from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')

if TYPE_CHECKING:
    from Live.LomObject import LomObject



class FloatVector(Vector[float]):
    """A simple container for returning floats from Live."""

    def append(self, value: float | None, /) -> None:
        ...

    def extend(self, values: Iterable[float] | None, /) -> None:
        ...

class IntU64Vector(Vector[int]):
    """A simple container for returning unsigned long integers from Live."""

    def append(self, value: int | None, /) -> None:
        ...

    def extend(self, values: Iterable[int] | None, /) -> None:
        ...

class IntVector(Vector[int]):
    """A simple container for returning integers from Live."""

    def append(self, value: int | None, /) -> None:
        ...

    def extend(self, values: Iterable[int] | None, /) -> None:
        ...

class LimitationError(Exception): ...

class ObjectVector(Vector[object]):
    """A simple read only container for returning python objects."""

    def append(self, value: Any | None, /) -> None:
        ...

    def extend(self, values: Iterable[object] | None, /) -> None:
        ...

class StringVector(Vector[str]):
    """A simple container for returning strings from Live."""

    def append(self, value: str | None, /) -> None:
        ...

    def extend(self, values: Iterable[str] | None, /) -> None:
        ...

class Text:
    """A translatable, immutable string."""

    @property
    def text(self) -> str:
        ...

class Timer:
    """A timer that will trigger a callback after a certain inverval. The timer can be repeated and will trigger the callback every interval. Errors in the callback will stop the timer."""

    def __init__(self, callback: object, interval: int, repeat: bool = False, start: bool = False) -> None: ...

    def restart(self) -> None:
        ...

    @property
    def running(self) -> bool:
        ...

    def start(self) -> None:
        ...

    def stop(self) -> None:
        ...

class Vector(Generic[T]):
    """A simple read only container for returning objects from Live."""

    def __iter__(self) -> Iterator[T]: ...

    @overload
    def __getitem__(self, index: int) -> T: ...

    @overload
    def __getitem__(self, index: slice) -> Vector[T]: ...

    def __getitem__(self, index: int | slice) -> T | Vector[T]: ...

    def __len__(self) -> int: ...

    def __contains__(self, value: object) -> bool: ...

    def __bool__(self) -> bool: ...

    def append(self, value: LomObject | None, /) -> None:
        ...

    def extend(self, values: Iterable[LomObject] | None, /) -> None:
        ...

def get_text(classname: str | None, textname: str | None, /) -> Text:
    """Retrieves the (translated) Text identified by `classname` and `textname`."""
    ...

def log(string: str | None, /) -> None:
    ...

def subst_args(text: Text | None, arg1: str = '', arg2: str = '', arg3: str = '', arg4: str = '', arg5: str = '', /) -> str:
    ...

__all__ = ['FloatVector', 'IntU64Vector', 'IntVector', 'LimitationError', 'ObjectVector', 'StringVector', 'Text', 'Timer', 'Vector', 'get_text', 'log', 'subst_args']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable


class ProgressDialog:
    """A modal dialog showing a message and a progress animation."""

    def end_modal_loop(self) -> None:
        ...

    def run_in_modal_loop(self) -> None:
        ...

    def set_status_message(self, msg: str | None, /) -> None:
        ...

class PythonLicensingBridge:
    """Interface to the internal licensing services."""

    def authorize_with_sassafras(self) -> None:
        ...

    @property
    def base_product_id(self) -> str:
        """Returns Live's current base product ID."""
        ...

    def create_new_live_set(self) -> None:
        """Creates a new live set and discards unsaved changes."""
        ...

    def deauthenticate_user(self) -> None:
        """Deletes the current session ID."""
        ...

    def get_progress_dialog(self) -> ProgressDialog:
        """Retrieves an instance of ProgressDialog."""
        ...

    def get_session_id(self) -> str:
        """Retrieve stored session ID."""
        ...

    def get_startup_dialog(self, authorize_callable: Callable | None, authorize_later_callable: Callable | None, /) -> StartupDialog:
        """Retrieves an instance of the startup dialog with the passed callables connected to its buttons."""
        ...

    def get_trial_time_left(self) -> str:
        """Returns remaining time on a trial as a formatted string."""
        ...

    @property
    def in_sassafras_mode(self) -> bool:
        ...

    def invoke_pack_installation_callback(self) -> None:
        """Call package installation callback."""
        ...

    @property
    def license_must_match_variant(self) -> bool:
        """Returns a bool indicating if we require the license information returned by the server to match the variant of Live."""
        ...

    def load_and_convert_legacy_unlock_cfg(self) -> dict:
        """Loads the Unlock.cfg file and returns either an empty dict or one that can be converted to an UnlockData object."""
        ...

    def process_license_response(self, license_response_lines: list[str] | None, /) -> UnlockStatus:
        """Processes a list of strings, each representing a server response to a product authorization."""
        ...

    def process_trial_response(self, trial_response_line: str | None, /) -> bool:
        """Process the server's response to a Trial authorization."""
        ...

    @property
    def random_number_for_trial_authorization(self) -> int:
        """Returns the integer to send along with the Trial authorization request. This same integer will be checked for in `process_trial_response` (and then changed)."""
        ...

    def request_exit(self, exit_code: int = 0, /) -> None:
        ...

    def save_current_set(self) -> None:
        """Saves the current Live session."""
        ...

    @property
    def set_has_unsaved_changes(self) -> bool:
        """Returns true if the set has unsaved changes."""
        ...

    def set_network_timer(self, callback: Callable | None, interval_in_ms: int | None, /) -> None:
        """Starts or stops a timer meant for driving network operations. Pass None as callback to stop the timer. If any callback invocation raises an exception, the timer is stopped."""
        ...

    def store_session_id(self, session_id: str | None, /) -> None:
        """Securely stores the user's session ID (aka cookie, aka credentials)."""
        ...

class StartupDialog:
    """Serves as an entry point for the user to authorize Live on first launch."""

    def end_modal_loop(self) -> None:
        ...

    def run_in_modal_loop(self, show_only_offline_auth_instructions: bool | None, /) -> None:
        ...

    def set_notification_message(self, notification_text: str | None, show_progress_bar: bool | None, /) -> None:
        ...

class TrialContext(int):
    SAVE: int = 0
    FORCE_UPDATE: int = 2
    STARTUP: int = 3

class UnlockStatus:
    """Returns relevant information after unlock"""

    def __init__(self) -> None: ...

    @property
    def authorization_deactivated(self) -> bool:
        ...

    @property
    def authorization_expired(self) -> bool:
        ...

    @property
    def has_max_unlock_products(self) -> bool:
        ...

    @property
    def temp_demo_mode(self) -> bool:
        ...

    @property
    def time_limited(self) -> bool:
        ...

    @property
    def unlock_error(self) -> bool:
        ...

    @property
    def unlocked(self) -> bool:
        ...

def authorization_clock_days_ahead() -> int:
    """Advances the current date by the number of days specified by _AuthClockDaysAhead"""
    ...

def get_authorization_page_url(reauthorize: bool | None, is_trial: bool | None, /) -> str:
    """Retrieves the appopriate URL on ableton.com where the unser can initiate the authorization."""
    ...

def get_purchase_live_url() -> str:
    """Returns the environment-aware purchase URL for purchasing Live licenses"""
    ...

def get_services_url() -> str:
    """Returns the URL against which service calls (e.g. for authorization) can be performed."""
    ...

def get_unlock_dir() -> tuple[str, bool]:
    """Returns a tuple containing the unlock file directory and a flag indicating if the unlock file is in the system domain."""
    ...

def launch_web_browser(url: str | None, /) -> None:
    """Opens a web browser at the specified URL on the user's computer."""
    ...

__all__ = ['ProgressDialog', 'PythonLicensingBridge', 'StartupDialog', 'TrialContext', 'UnlockStatus', 'authorization_clock_days_ahead', 'get_authorization_page_url', 'get_purchase_live_url', 'get_services_url', 'get_unlock_dir', 'launch_web_browser']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import StringVector
    from Live.Device import ATimeableValueVector, Device, DeviceType
    from Live.Track import Track



class DriftDevice(Device):
    """This class represents a Drift device."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_mod_matrix_filter_source_1_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_filter_source_1_index" has changed.
        """
        ...

    def add_mod_matrix_filter_source_2_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_filter_source_2_index" has changed.
        """
        ...

    def add_mod_matrix_lfo_source_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_lfo_source_index" has changed.
        """
        ...

    def add_mod_matrix_pitch_source_1_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_pitch_source_1_index" has changed.
        """
        ...

    def add_mod_matrix_pitch_source_2_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_pitch_source_2_index" has changed.
        """
        ...

    def add_mod_matrix_shape_source_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_shape_source_index" has changed.
        """
        ...

    def add_mod_matrix_source_1_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_source_1_index" has changed.
        """
        ...

    def add_mod_matrix_source_2_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_source_2_index" has changed.
        """
        ...

    def add_mod_matrix_source_3_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_source_3_index" has changed.
        """
        ...

    def add_mod_matrix_target_1_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_target_1_index" has changed.
        """
        ...

    def add_mod_matrix_target_2_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_target_2_index" has changed.
        """
        ...

    def add_mod_matrix_target_3_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "mod_matrix_target_3_index" has changed.
        """
        ...

    def add_pitch_bend_range_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "pitch_bend_range" has changed.
        """
        ...

    def add_voice_count_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "voice_count_index" has changed.
        """
        ...

    def add_voice_mode_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "voice_mode_index" has changed.
        """
        ...

    @property
    def can_compare_ab(self) -> bool:
        """Returns true if the Device has the capability to AB compare."""
        ...

    @property
    def can_have_chains(self) -> bool:
        """Returns true if the device is a rack."""
        ...

    @property
    def can_have_drum_pads(self) -> bool:
        """Returns true if the device is a drum rack."""
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the Device."""
        ...

    @property
    def class_display_name(self) -> str:
        """Return const access to the name of the device's class name as displayed in Live's browser and device chain"""
        ...

    @property
    def class_name(self) -> str:
        """Return const access to the name of the device's class."""
        ...

    @property
    def is_active(self) -> bool:
        """Return const access to whether this device is active. This will be false bothwhen the device is off and when it's inside a rack device which is off."""
        ...

    @property
    def is_using_compare_preset_b(self) -> bool:
        """Returns whether the Device has loaded the preset in compare slot B. Only relevant if can_compare_ab, otherwise errors."""
        ...

    @is_using_compare_preset_b.setter
    def is_using_compare_preset_b(self, value: bool) -> None: ...

    @property
    def latency_in_ms(self) -> float:
        """Returns the latency of the device in ms."""
        ...

    @property
    def latency_in_samples(self) -> int:
        """Returns the latency of the device in samples."""
        ...

    @property
    def mod_matrix_filter_source_1_index(self) -> int:
        """Return the filter mod source 1 index"""
        ...

    @mod_matrix_filter_source_1_index.setter
    def mod_matrix_filter_source_1_index(self, value: int) -> None: ...

    def mod_matrix_filter_source_1_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_filter_source_1_index".
        """
        ...

    @property
    def mod_matrix_filter_source_1_list(self) -> StringVector:
        """Return the filter mod source 1 list"""
        ...

    @property
    def mod_matrix_filter_source_2_index(self) -> int:
        """Return the filter mod source 2 index"""
        ...

    @mod_matrix_filter_source_2_index.setter
    def mod_matrix_filter_source_2_index(self, value: int) -> None: ...

    def mod_matrix_filter_source_2_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_filter_source_2_index".
        """
        ...

    @property
    def mod_matrix_filter_source_2_list(self) -> StringVector:
        """Return the filter mod source 2 list"""
        ...

    @property
    def mod_matrix_lfo_source_index(self) -> int:
        """Return the lfo mod source index"""
        ...

    @mod_matrix_lfo_source_index.setter
    def mod_matrix_lfo_source_index(self, value: int) -> None: ...

    def mod_matrix_lfo_source_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_lfo_source_index".
        """
        ...

    @property
    def mod_matrix_lfo_source_list(self) -> StringVector:
        """Return the lfo mod source list"""
        ...

    @property
    def mod_matrix_pitch_source_1_index(self) -> int:
        """Return the pitch mod source 1 index"""
        ...

    @mod_matrix_pitch_source_1_index.setter
    def mod_matrix_pitch_source_1_index(self, value: int) -> None: ...

    def mod_matrix_pitch_source_1_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_pitch_source_1_index".
        """
        ...

    @property
    def mod_matrix_pitch_source_1_list(self) -> StringVector:
        """Return the pitch mod source 1 list"""
        ...

    @property
    def mod_matrix_pitch_source_2_index(self) -> int:
        """Return the pitch mod source 2 index"""
        ...

    @mod_matrix_pitch_source_2_index.setter
    def mod_matrix_pitch_source_2_index(self, value: int) -> None: ...

    def mod_matrix_pitch_source_2_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_pitch_source_2_index".
        """
        ...

    @property
    def mod_matrix_pitch_source_2_list(self) -> StringVector:
        """Return the pitch mod source 2 list"""
        ...

    @property
    def mod_matrix_shape_source_index(self) -> int:
        """Return the shape mod source index"""
        ...

    @mod_matrix_shape_source_index.setter
    def mod_matrix_shape_source_index(self, value: int) -> None: ...

    def mod_matrix_shape_source_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_shape_source_index".
        """
        ...

    @property
    def mod_matrix_shape_source_list(self) -> StringVector:
        """Return the shape mod source list"""
        ...

    @property
    def mod_matrix_source_1_index(self) -> int:
        """Return the custom mod source 1 index"""
        ...

    @mod_matrix_source_1_index.setter
    def mod_matrix_source_1_index(self, value: int) -> None: ...

    def mod_matrix_source_1_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_source_1_index".
        """
        ...

    @property
    def mod_matrix_source_1_list(self) -> StringVector:
        """Return the custom mod source 1 list"""
        ...

    @property
    def mod_matrix_source_2_index(self) -> int:
        """Return the custom mod source 2 index"""
        ...

    @mod_matrix_source_2_index.setter
    def mod_matrix_source_2_index(self, value: int) -> None: ...

    def mod_matrix_source_2_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_source_2_index".
        """
        ...

    @property
    def mod_matrix_source_2_list(self) -> StringVector:
        """Return the custom mod source 2 list"""
        ...

    @property
    def mod_matrix_source_3_index(self) -> int:
        """Return the custom mod source 3 index"""
        ...

    @mod_matrix_source_3_index.setter
    def mod_matrix_source_3_index(self, value: int) -> None: ...

    def mod_matrix_source_3_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_source_3_index".
        """
        ...

    @property
    def mod_matrix_source_3_list(self) -> StringVector:
        """Return the custom mod source 3 list"""
        ...

    @property
    def mod_matrix_target_1_index(self) -> int:
        """Return the custom mod target 1 index"""
        ...

    @mod_matrix_target_1_index.setter
    def mod_matrix_target_1_index(self, value: int) -> None: ...

    def mod_matrix_target_1_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_target_1_index".
        """
        ...

    @property
    def mod_matrix_target_1_list(self) -> StringVector:
        """Return the custom mod target 1 list"""
        ...

    @property
    def mod_matrix_target_2_index(self) -> int:
        """Return the custom mod target 2 index"""
        ...

    @mod_matrix_target_2_index.setter
    def mod_matrix_target_2_index(self, value: int) -> None: ...

    def mod_matrix_target_2_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_target_2_index".
        """
        ...

    @property
    def mod_matrix_target_2_list(self) -> StringVector:
        """Return the custom mod target 2 list"""
        ...

    @property
    def mod_matrix_target_3_index(self) -> int:
        """Return the custom mod target 3 index"""
        ...

    @mod_matrix_target_3_index.setter
    def mod_matrix_target_3_index(self, value: int) -> None: ...

    def mod_matrix_target_3_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "mod_matrix_target_3_index".
        """
        ...

    @property
    def mod_matrix_target_3_list(self) -> StringVector:
        """Return the custom mod target 3 list"""
        ...

    @property
    def name(self) -> str:
        """Return access to the name of the device."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    @property
    def parameters(self) -> ATimeableValueVector:
        """Const access to the list of available automatable parameters for this device."""
        ...

    @property
    def pitch_bend_range(self) -> int:
        """Return the Pitch Bend Range"""
        ...

    @pitch_bend_range.setter
    def pitch_bend_range(self, value: int) -> None: ...

    def pitch_bend_range_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "pitch_bend_range".
        """
        ...

    def remove_mod_matrix_filter_source_1_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_filter_source_1_index".
        """
        ...

    def remove_mod_matrix_filter_source_2_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_filter_source_2_index".
        """
        ...

    def remove_mod_matrix_lfo_source_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_lfo_source_index".
        """
        ...

    def remove_mod_matrix_pitch_source_1_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_pitch_source_1_index".
        """
        ...

    def remove_mod_matrix_pitch_source_2_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_pitch_source_2_index".
        """
        ...

    def remove_mod_matrix_shape_source_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_shape_source_index".
        """
        ...

    def remove_mod_matrix_source_1_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_source_1_index".
        """
        ...

    def remove_mod_matrix_source_2_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_source_2_index".
        """
        ...

    def remove_mod_matrix_source_3_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_source_3_index".
        """
        ...

    def remove_mod_matrix_target_1_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_target_1_index".
        """
        ...

    def remove_mod_matrix_target_2_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_target_2_index".
        """
        ...

    def remove_mod_matrix_target_3_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "mod_matrix_target_3_index".
        """
        ...

    def remove_pitch_bend_range_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "pitch_bend_range".
        """
        ...

    def remove_voice_count_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "voice_count_index".
        """
        ...

    def remove_voice_mode_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "voice_mode_index".
        """
        ...

    @property
    def type(self) -> DeviceType:
        """Return the type of the device."""
        ...

    @property
    def view(self) -> Device.View:
        """Representing the view aspects of a device."""
        ...

    @property
    def voice_count_index(self) -> int:
        """Return the voice count index"""
        ...

    @voice_count_index.setter
    def voice_count_index(self, value: int) -> None: ...

    def voice_count_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "voice_count_index".
        """
        ...

    @property
    def voice_count_list(self) -> StringVector:
        """Return the voice count list"""
        ...

    @property
    def voice_mode_index(self) -> int:
        """Return the voice mode index"""
        ...

    @voice_mode_index.setter
    def voice_mode_index(self, value: int) -> None: ...

    def voice_mode_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "voice_mode_index".
        """
        ...

    @property
    def voice_mode_list(self) -> StringVector:
        """Return the voice mode list"""
        ...

__all__ = ['DriftDevice']



from . import Application
from . import Base
from . import Browser
from . import CcControlDevice
from . import Chain
from . import ChainMixerDevice
from . import Clip
from . import ClipSlot
from . import CompressorDevice
from . import Conversions
from . import Device
from . import DeviceIO
from . import DeviceParameter
from . import DriftDevice
from . import DrumCellDevice
from . import DrumChain
from . import DrumPad
from . import Envelope
from . import Eq8Device
from . import Groove
from . import GroovePool
from . import HybridReverbDevice
from . import Licensing
from . import Listener
from . import LomObject
from . import LooperDevice
from . import MaxDevice
from . import MeldDevice
from . import MidiMap
from . import MixerDevice
from . import PluginDevice
from . import RackDevice
from . import RoarDevice
from . import Sample
from . import Scene
from . import ShifterDevice
from . import SimplerDevice
from . import Song
from . import SpectralResonatorDevice
from . import TakeLane
from . import Track
from . import TuningSystem
from . import WavetableDevice

__all__ = ['Application', 'Base', 'Browser', 'CcControlDevice', 'Chain', 'ChainMixerDevice', 'Clip', 'ClipSlot', 'CompressorDevice', 'Conversions', 'Device', 'DeviceIO', 'DeviceParameter', 'DriftDevice', 'DrumCellDevice', 'DrumChain', 'DrumPad', 'Envelope', 'Eq8Device', 'Groove', 'GroovePool', 'HybridReverbDevice', 'Licensing', 'Listener', 'LomObject', 'LooperDevice', 'MaxDevice', 'MeldDevice', 'MidiMap', 'MixerDevice', 'PluginDevice', 'RackDevice', 'RoarDevice', 'Sample', 'Scene', 'ShifterDevice', 'SimplerDevice', 'Song', 'SpectralResonatorDevice', 'TakeLane', 'Track', 'TuningSystem', 'WavetableDevice']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import StringVector
    from Live.Clip import Clip
    from Live.ClipSlot import ClipSlot
    from Live.Device import ATimeableValueVector, Device, DeviceType
    from Live.Track import Track



class LooperDevice(Device):
    """This class represents a Looper device."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_loop_length_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "loop_length" has changed.
        """
        ...

    def add_overdub_after_record_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "overdub_after_record" has changed.
        """
        ...

    def add_record_length_index_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "record_length_index" has changed.
        """
        ...

    def add_tempo_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "tempo" has changed.
        """
        ...

    @property
    def can_compare_ab(self) -> bool:
        """Returns true if the Device has the capability to AB compare."""
        ...

    @property
    def can_have_chains(self) -> bool:
        """Returns true if the device is a rack."""
        ...

    @property
    def can_have_drum_pads(self) -> bool:
        """Returns true if the device is a drum rack."""
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the Device."""
        ...

    @property
    def class_display_name(self) -> str:
        """Return const access to the name of the device's class name as displayed in Live's browser and device chain"""
        ...

    @property
    def class_name(self) -> str:
        """Return const access to the name of the device's class."""
        ...

    def clear(self) -> None:
        """Erase Looper's recorded content."""
        ...

    def double_length(self) -> None:
        """Double the length of Looper's buffer."""
        ...

    def double_speed(self) -> None:
        """Double the speed of Looper's playback."""
        ...

    def export_to_clip_slot(self, clip_slot: ClipSlot | None, /) -> None:
        """Export Looper's content to a Session Clip Slot."""
        ...

    def half_length(self) -> None:
        """Halve the length of Looper's buffer."""
        ...

    def half_speed(self) -> None:
        """Halve the speed of Looper's playback."""
        ...

    @property
    def is_active(self) -> bool:
        """Return const access to whether this device is active. This will be false bothwhen the device is off and when it's inside a rack device which is off."""
        ...

    @property
    def is_using_compare_preset_b(self) -> bool:
        """Returns whether the Device has loaded the preset in compare slot B. Only relevant if can_compare_ab, otherwise errors."""
        ...

    @is_using_compare_preset_b.setter
    def is_using_compare_preset_b(self, value: bool) -> None: ...

    @property
    def latency_in_ms(self) -> float:
        """Returns the latency of the device in ms."""
        ...

    @property
    def latency_in_samples(self) -> int:
        """Returns the latency of the device in samples."""
        ...

    @property
    def loop_length(self) -> float:
        """The length of Looper's buffer."""
        ...

    def loop_length_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "loop_length".
        """
        ...

    @property
    def name(self) -> str:
        """Return access to the name of the device."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    def overdub(self) -> None:
        """Play back while adding additional layers of incoming audio."""
        ...

    @property
    def overdub_after_record(self) -> bool:
        """If true, Looper will switch to overdub after recording, when recording a fixed number of bars. Otherwise, the switch will be to playback without overdubbing."""
        ...

    @overdub_after_record.setter
    def overdub_after_record(self, value: bool) -> None: ...

    def overdub_after_record_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "overdub_after_record".
        """
        ...

    @property
    def parameters(self) -> ATimeableValueVector:
        """Const access to the list of available automatable parameters for this device."""
        ...

    def play(self) -> None:
        """Play back without overdubbing."""
        ...

    def record(self) -> None:
        """Record incoming audio."""
        ...

    @property
    def record_length_index(self) -> int:
        """Access to the Record Length chooser entry index."""
        ...

    @record_length_index.setter
    def record_length_index(self, value: int) -> None: ...

    def record_length_index_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "record_length_index".
        """
        ...

    @property
    def record_length_list(self) -> StringVector:
        """Read-only access to the list of Record Length chooser entry strings."""
        ...

    def remove_loop_length_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "loop_length".
        """
        ...

    def remove_overdub_after_record_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "overdub_after_record".
        """
        ...

    def remove_record_length_index_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "record_length_index".
        """
        ...

    def remove_tempo_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "tempo".
        """
        ...

    def stop(self) -> None:
        """Stop Looper's playback."""
        ...

    @property
    def tempo(self) -> float:
        """The tempo of Looper's buffer."""
        ...

    def tempo_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "tempo".
        """
        ...

    @property
    def type(self) -> DeviceType:
        """Return the type of the device."""
        ...

    def undo(self) -> None:
        """Erase everything that was recorded since the last time Overdub was enabled. Calling a second time will restore the material erased by the previous undooperation."""
        ...

    @property
    def view(self) -> Device.View:
        """Representing the view aspects of a device."""
        ...

__all__ = ['LooperDevice']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Generic, Iterable, Iterator, TypeVar, overload

T = TypeVar('T')

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.DeviceParameter import DeviceParameter
    from Live.LomObject import LomObject
    from Live.Track import Track



class Device(LomObject):
    """This class represents a MIDI or Audio DSP-Device in Live."""

    class View(LomObject):
        """Representing the view aspects of a device."""

        @property
        def _live_ptr(self) -> int:
            ...

        def add_is_collapsed_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "is_collapsed" has changed.
            """
            ...

        @property
        def canonical_parent(self) -> Device:
            """Get the canonical parent of the View."""
            ...

        @property
        def is_collapsed(self) -> bool:
            """Get/Set/Listen if the device is shown collapsed in the device chain."""
            ...

        @is_collapsed.setter
        def is_collapsed(self, value: bool) -> None: ...

        def is_collapsed_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "is_collapsed".
            """
            ...

        def remove_is_collapsed_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "is_collapsed".
            """
            ...

    @property
    def _live_ptr(self) -> int:
        ...

    def add_is_active_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_active" has changed.
        """
        ...

    def add_is_using_compare_preset_b_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_using_compare_preset_b" has changed.
        """
        ...

    def add_latency_in_ms_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "latency_in_ms" has changed.
        """
        ...

    def add_latency_in_samples_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "latency_in_samples" has changed.
        """
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    def add_parameters_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "parameters" has changed.
        """
        ...

    @property
    def can_compare_ab(self) -> bool:
        """Returns true if the Device has the capability to AB compare."""
        ...

    @property
    def can_have_chains(self) -> bool:
        """Returns true if the device is a rack."""
        ...

    @property
    def can_have_drum_pads(self) -> bool:
        """Returns true if the device is a drum rack."""
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the Device."""
        ...

    @property
    def class_display_name(self) -> str:
        """Return const access to the name of the device's class name as displayed in Live's browser and device chain"""
        ...

    @property
    def class_name(self) -> str:
        """Return const access to the name of the device's class."""
        ...

    @property
    def is_active(self) -> bool:
        """Return const access to whether this device is active. This will be false bothwhen the device is off and when it's inside a rack device which is off."""
        ...

    def is_active_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_active".
        """
        ...

    @property
    def is_using_compare_preset_b(self) -> bool:
        """Returns whether the Device has loaded the preset in compare slot B. Only relevant if can_compare_ab, otherwise errors."""
        ...

    @is_using_compare_preset_b.setter
    def is_using_compare_preset_b(self, value: bool) -> None: ...

    def is_using_compare_preset_b_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_using_compare_preset_b".
        """
        ...

    @property
    def latency_in_ms(self) -> float:
        """Returns the latency of the device in ms."""
        ...

    def latency_in_ms_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "latency_in_ms".
        """
        ...

    @property
    def latency_in_samples(self) -> int:
        """Returns the latency of the device in samples."""
        ...

    def latency_in_samples_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "latency_in_samples".
        """
        ...

    @property
    def name(self) -> str:
        """Return access to the name of the device."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    @property
    def parameters(self) -> ATimeableValueVector:
        """Const access to the list of available automatable parameters for this device."""
        ...

    def parameters_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "parameters".
        """
        ...

    def remove_is_active_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_active".
        """
        ...

    def remove_is_using_compare_preset_b_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_using_compare_preset_b".
        """
        ...

    def remove_latency_in_ms_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "latency_in_ms".
        """
        ...

    def remove_latency_in_samples_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "latency_in_samples".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

    def remove_parameters_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "parameters".
        """
        ...

    def save_preset_to_compare_ab_slot(self) -> None:
        """Saves the current state of the device to the compare AB slot. Only relevant if can_compare_ab, otherwise throws."""
        ...

    def store_chosen_bank(self, script_index: int | None, bank_index: int | None, /) -> None:
        """Set the selected bank in the device for persistency."""
        ...

    @property
    def type(self) -> DeviceType:
        """Return the type of the device."""
        ...

    @property
    def view(self) -> View:
        """Representing the view aspects of a device."""
        ...

class ATimeableValueVector(Vector[DeviceParameter]):

    def append(self, value: DeviceParameter | None, /) -> None:
        ...

    def extend(self, values: Iterable[DeviceParameter] | None, /) -> None:
        ...

class DeviceType(int):
    """The type of the device."""
    undefined: int = 0
    instrument: int = 1
    audio_effect: int = 2
    midi_effect: int = 4

__all__ = ['Device', 'ATimeableValueVector', 'DeviceType']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.Clip import Clip
    from Live.LomObject import LomObject
    from Live.Track import Track



class TakeLane(LomObject):
    """This class represents a take lane in Live."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_arrangement_clips_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "arrangement_clips" has changed.
        """
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    @property
    def arrangement_clips(self) -> Vector[Clip]:
        """Read-only access to the arrangement clips in the take lane."""
        ...

    def arrangement_clips_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "arrangement_clips".
        """
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the take lane."""
        ...

    def create_audio_clip(self, file_path: str | None, start_time: float | None, /) -> Clip:
        """
        Creates an audio clip referencing the file at the given path and inserts it into the arrangement at the specified time.
        Throws an error when called on a non-audio or a frozen track, when the specified time is outside the [0., 1576800.] range, when the track is currently being recorded into, or when the path doesn't point to a valid audio file.
        """
        ...

    def create_midi_clip(self, start_time: float | None, length: float | None, /) -> Clip:
        """
        Creates an empty MIDI clip and inserts it into the arrangement at the specified time.
        Throws an error when called on a non-MIDI track or a frozen track, when the specified time is outside the [0., 1576800.] range, or when the track is currently being recorded into.
        """
        ...

    @property
    def name(self) -> str:
        """Read/write access to the name of the TakeLane, as visible in the take lane header."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    def remove_arrangement_clips_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "arrangement_clips".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

__all__ = ['TakeLane']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Clip import WarpMarkerVector
    from Live.Envelope import Envelope
    from Live.LomObject import LomObject
    from Live.SimplerDevice import SimplerDevice



class Sample(LomObject):
    """This class represents a sample file loaded into a Simpler instance."""

    @property
    def _live_ptr(self) -> int:
        ...

    def add_beats_granulation_resolution_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "beats_granulation_resolution" has changed.
        """
        ...

    def add_beats_transient_envelope_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "beats_transient_envelope" has changed.
        """
        ...

    def add_beats_transient_loop_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "beats_transient_loop_mode" has changed.
        """
        ...

    def add_complex_pro_envelope_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "complex_pro_envelope" has changed.
        """
        ...

    def add_complex_pro_formants_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "complex_pro_formants" has changed.
        """
        ...

    def add_end_marker_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "end_marker" has changed.
        """
        ...

    def add_file_path_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "file_path" has changed.
        """
        ...

    def add_gain_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "gain" has changed.
        """
        ...

    def add_slices_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "slices" has changed.
        """
        ...

    def add_slicing_beat_division_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "slicing_beat_division" has changed.
        """
        ...

    def add_slicing_region_count_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "slicing_region_count" has changed.
        """
        ...

    def add_slicing_sensitivity_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "slicing_sensitivity" has changed.
        """
        ...

    def add_slicing_style_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "slicing_style" has changed.
        """
        ...

    def add_start_marker_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "start_marker" has changed.
        """
        ...

    def add_texture_flux_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "texture_flux" has changed.
        """
        ...

    def add_texture_grain_size_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "texture_grain_size" has changed.
        """
        ...

    def add_tones_grain_size_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "tones_grain_size" has changed.
        """
        ...

    def add_warp_markers_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "warp_markers" has changed.
        """
        ...

    def add_warp_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "warp_mode" has changed.
        """
        ...

    def add_warping_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "warping" has changed.
        """
        ...

    def beat_to_sample_time(self, beat_time: float | None, /) -> float:
        """Converts the given beat time to sample time. Raises an error if the sample is not warped."""
        ...

    @property
    def beats_granulation_resolution(self) -> int:
        """Access to the Granulation Resolution parameter in Beats Warp Mode."""
        ...

    @beats_granulation_resolution.setter
    def beats_granulation_resolution(self, value: int) -> None: ...

    def beats_granulation_resolution_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "beats_granulation_resolution".
        """
        ...

    @property
    def beats_transient_envelope(self) -> float:
        """Access to the Transient Envelope parameter in Beats Warp Mode."""
        ...

    @beats_transient_envelope.setter
    def beats_transient_envelope(self, value: float) -> None: ...

    def beats_transient_envelope_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "beats_transient_envelope".
        """
        ...

    @property
    def beats_transient_loop_mode(self) -> int:
        """Access to the Transient Loop Mode parameter in Beats Warp Mode."""
        ...

    @beats_transient_loop_mode.setter
    def beats_transient_loop_mode(self, value: int) -> None: ...

    def beats_transient_loop_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "beats_transient_loop_mode".
        """
        ...

    @property
    def canonical_parent(self) -> SimplerDevice:
        """Access to the sample's canonical parent."""
        ...

    def clear_slices(self) -> None:
        """Clears all slices created in Simpler's manual mode."""
        ...

    @property
    def complex_pro_envelope(self) -> float:
        """Access to the Envelope parameter in Complex Pro Mode."""
        ...

    @complex_pro_envelope.setter
    def complex_pro_envelope(self, value: float) -> None: ...

    def complex_pro_envelope_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "complex_pro_envelope".
        """
        ...

    @property
    def complex_pro_formants(self) -> float:
        """Access to the Formants parameter in Complex Pro Warp Mode."""
        ...

    @complex_pro_formants.setter
    def complex_pro_formants(self, value: float) -> None: ...

    def complex_pro_formants_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "complex_pro_formants".
        """
        ...

    @property
    def end_marker(self) -> int:
        """Access to the position of the sample's end marker."""
        ...

    @end_marker.setter
    def end_marker(self, value: int) -> None: ...

    def end_marker_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "end_marker".
        """
        ...

    @property
    def file_path(self) -> str:
        """Get the path of the sample file."""
        ...

    def file_path_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "file_path".
        """
        ...

    @property
    def gain(self) -> float:
        """Access to the sample gain."""
        ...

    @gain.setter
    def gain(self, value: float) -> None: ...

    def gain_display_string(self) -> str:
        """Get the gain's display value as a string."""
        ...

    def gain_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "gain".
        """
        ...

    def insert_slice(self, slice_time: int | None, /) -> None:
        """Add a slice point at the provided time if there is none."""
        ...

    @property
    def length(self) -> int:
        """Get the length of the sample file in sample frames."""
        ...

    def move_slice(self, old_time: int | None, new_time: int | None, /) -> int:
        """Move the slice point at the provided time."""
        ...

    def remove_beats_granulation_resolution_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "beats_granulation_resolution".
        """
        ...

    def remove_beats_transient_envelope_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "beats_transient_envelope".
        """
        ...

    def remove_beats_transient_loop_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "beats_transient_loop_mode".
        """
        ...

    def remove_complex_pro_envelope_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "complex_pro_envelope".
        """
        ...

    def remove_complex_pro_formants_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "complex_pro_formants".
        """
        ...

    def remove_end_marker_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "end_marker".
        """
        ...

    def remove_file_path_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "file_path".
        """
        ...

    def remove_gain_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "gain".
        """
        ...

    def remove_slice(self, slice_time: int | None, /) -> None:
        """Remove the slice point at the provided time if there is one."""
        ...

    def remove_slices_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "slices".
        """
        ...

    def remove_slicing_beat_division_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "slicing_beat_division".
        """
        ...

    def remove_slicing_region_count_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "slicing_region_count".
        """
        ...

    def remove_slicing_sensitivity_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "slicing_sensitivity".
        """
        ...

    def remove_slicing_style_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "slicing_style".
        """
        ...

    def remove_start_marker_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "start_marker".
        """
        ...

    def remove_texture_flux_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "texture_flux".
        """
        ...

    def remove_texture_grain_size_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "texture_grain_size".
        """
        ...

    def remove_tones_grain_size_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "tones_grain_size".
        """
        ...

    def remove_warp_markers_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "warp_markers".
        """
        ...

    def remove_warp_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "warp_mode".
        """
        ...

    def remove_warping_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "warping".
        """
        ...

    def reset_slices(self) -> None:
        """Resets all edited slices to their original positions."""
        ...

    @property
    def sample_rate(self) -> float:
        """Access to the audio sample rate of the sample."""
        ...

    def sample_to_beat_time(self, sample_time: float | None, /) -> float:
        """Converts the given sample time to beat time. Raises an error if the sample is not warped."""
        ...

    @property
    def slices(self) -> tuple[int, ...]:
        """Access to the list of slice points in sample time in the sample."""
        ...

    def slices_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "slices".
        """
        ...

    @property
    def slicing_beat_division(self) -> int:
        """Access to sample's slicing step size."""
        ...

    @slicing_beat_division.setter
    def slicing_beat_division(self, value: int) -> None: ...

    def slicing_beat_division_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "slicing_beat_division".
        """
        ...

    @property
    def slicing_region_count(self) -> int:
        """Access to sample's slicing split count."""
        ...

    @slicing_region_count.setter
    def slicing_region_count(self, value: int) -> None: ...

    def slicing_region_count_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "slicing_region_count".
        """
        ...

    @property
    def slicing_sensitivity(self) -> float:
        """
        Access to sample's slicing sensitivity whose sensitivity is in between 0.0 and 1.0.
        The higher the sensitivity, the more slices will be available.
        """
        ...

    @slicing_sensitivity.setter
    def slicing_sensitivity(self, value: float) -> None: ...

    def slicing_sensitivity_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "slicing_sensitivity".
        """
        ...

    @property
    def slicing_style(self) -> int:
        """Access to sample's slicing style."""
        ...

    @slicing_style.setter
    def slicing_style(self, value: int) -> None: ...

    def slicing_style_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "slicing_style".
        """
        ...

    @property
    def start_marker(self) -> int:
        """Access to the position of the sample's start marker."""
        ...

    @start_marker.setter
    def start_marker(self, value: int) -> None: ...

    def start_marker_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "start_marker".
        """
        ...

    @property
    def texture_flux(self) -> float:
        """Access to the Flux parameter in Texture Warp Mode."""
        ...

    @texture_flux.setter
    def texture_flux(self, value: float) -> None: ...

    def texture_flux_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "texture_flux".
        """
        ...

    @property
    def texture_grain_size(self) -> float:
        """Access to the Grain Size parameter in Texture Warp Mode."""
        ...

    @texture_grain_size.setter
    def texture_grain_size(self, value: float) -> None: ...

    def texture_grain_size_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "texture_grain_size".
        """
        ...

    @property
    def tones_grain_size(self) -> float:
        """Access to the Grain Size parameter in Tones Warp Mode."""
        ...

    @tones_grain_size.setter
    def tones_grain_size(self, value: float) -> None: ...

    def tones_grain_size_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "tones_grain_size".
        """
        ...

    @property
    def warp_markers(self) -> WarpMarkerVector:
        """Get the warp markers for this sample."""
        ...

    def warp_markers_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "warp_markers".
        """
        ...

    @property
    def warp_mode(self) -> int:
        """Access to the sample's warp mode."""
        ...

    @warp_mode.setter
    def warp_mode(self, value: int) -> None: ...

    def warp_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "warp_mode".
        """
        ...

    @property
    def warping(self) -> bool:
        """Access to the sample's warping property."""
        ...

    @warping.setter
    def warping(self, value: bool) -> None: ...

    def warping_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "warping".
        """
        ...

class SlicingBeatDivision(int):
    sixteenth: int = 0
    sixteenth_triplett: int = 1
    eighth: int = 2
    eighth_triplett: int = 3
    quarter: int = 4
    quarter_triplett: int = 5
    half: int = 6
    half_triplett: int = 7
    one_bar: int = 8
    two_bars: int = 9
    four_bars: int = 10

class SlicingStyle(int):
    transient: int = 0
    beat: int = 1
    region: int = 2
    manual: int = 3

class TransientLoopMode(int):
    off: int = 0
    forward: int = 1
    alternate: int = 2

__all__ = ['Sample', 'SlicingBeatDivision', 'SlicingStyle', 'TransientLoopMode']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import StringVector
    from Live.Device import Device
    from Live.LomObject import LomObject



class DeviceParameter(LomObject):
    """
    This class represents a (automatable) parameter within a MIDI or
    Audio DSP-Device.
    """

    @property
    def _live_ptr(self) -> int:
        ...

    def add_automation_state_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "automation_state" has changed.
        """
        ...

    def add_display_value_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "display_value" has changed.
        """
        ...

    def add_name_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "name" has changed.
        """
        ...

    def add_state_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "state" has changed.
        """
        ...

    def add_value_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "value" has changed.
        """
        ...

    @property
    def automation_state(self) -> int:
        """Returns state of type AutomationState."""
        ...

    def automation_state_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "automation_state".
        """
        ...

    def begin_gesture(self) -> None:
        """Notify the begin of a modification of the parameter, when a sequence of modifications have to be consider a consistent group -- for Sexample, when recording automation."""
        ...

    @property
    def canonical_parent(self) -> Device:
        """Get the canonical parent of the device parameter."""
        ...

    @property
    def default_value(self) -> float:
        """
        Return the default value for this parameter.  A Default value is only
        available for non-quantized parameter types (see 'is_quantized').
        """
        ...

    @property
    def display_value(self) -> float:
        """
        Get/Set the current value (as visible in the GUI) this parameter.
        The value must be inside the min/max properties of this device.
        """
        ...

    @display_value.setter
    def display_value(self, value: float) -> None: ...

    def display_value_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "display_value".
        """
        ...

    def end_gesture(self) -> None:
        """Notify the end of a modification of the parameter. See begin_gesture."""
        ...

    @property
    def is_enabled(self) -> bool:
        """Returns false if the parameter has been macro mapped or disabled by Max."""
        ...

    @property
    def is_quantized(self) -> bool:
        """
        Returns True, if this value is a boolean or integer like switch.
        Non quantized values are continues float values.
        """
        ...

    @property
    def max(self) -> float:
        """
        Returns const access to the upper value of the allowed range for
        this parameter
        """
        ...

    @property
    def min(self) -> float:
        """
        Returns const access to the lower value of the allowed range for
        this parameter
        """
        ...

    @property
    def name(self) -> str:
        """
        Returns const access the name of this parameter, as visible in Lives
        automation choosers.
        """
        ...

    def name_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "name".
        """
        ...

    @property
    def original_name(self) -> str:
        """
        Returns const access the original name of this parameter, unaffected of
        any renamings.
        """
        ...

    def re_enable_automation(self) -> None:
        """Reenable automation for this parameter."""
        ...

    def remove_automation_state_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "automation_state".
        """
        ...

    def remove_display_value_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "display_value".
        """
        ...

    def remove_name_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "name".
        """
        ...

    def remove_state_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "state".
        """
        ...

    def remove_value_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "value".
        """
        ...

    @property
    def short_value_items(self) -> StringVector:
        """Return the list of possible values for this parameter. Like value_items, but prefers short value names if available. Raises an error if 'is_quantized' is False."""
        ...

    @property
    def state(self) -> int:
        """
        Returns the state of the parameter:
        - enabled - the parameter's value can be changed,
        - irrelevant - the parameter is enabled, but value changes will not take any effect until it gets enabled,
        - disabled - the parameter's value cannot be changed.
        """
        ...

    def state_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "state".
        """
        ...

    def str_for_value(self, value: float | None, /) -> str:
        """
        Return a string representation of the given value. To be used
        for display purposes only. This value can include characters like 'db' or
        'hz', depending on the type of the parameter.
        """
        ...

    @property
    def value(self) -> float:
        """
        Get/Set the current internal value of this parameter.
        The value must be inside the min/max properties of this device.
        """
        ...

    @value.setter
    def value(self, value: float) -> None: ...

    def value_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "value".
        """
        ...

    @property
    def value_items(self) -> StringVector:
        """Return the list of possible values for this parameter. Raises an error if 'is_quantized' is False."""
        ...

class AutomationState(int):
    none: int = 0
    playing: int = 1
    overridden: int = 2

class ParameterState(int):
    enabled: int = 0
    irrelevant: int = 1
    disabled: int = 2

__all__ = ['DeviceParameter', 'AutomationState', 'ParameterState']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.Chain import Chain
    from Live.Device import ATimeableValueVector, Device, DeviceType
    from Live.DeviceParameter import DeviceParameter
    from Live.DrumPad import DrumPad
    from Live.Track import Track



class RackDevice(Device):
    """This class represents a Rack device."""

    class View(Device.View):
        """Representing the view aspects of a rack device."""

        @property
        def _live_ptr(self) -> int:
            ...

        def add_drum_pads_scroll_position_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "drum_pads_scroll_position" has changed.
            """
            ...

        def add_is_showing_chain_devices_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "is_showing_chain_devices" has changed.
            """
            ...

        def add_selected_chain_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "selected_chain" has changed.
            """
            ...

        def add_selected_drum_pad_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "selected_drum_pad" has changed.
            """
            ...

        @property
        def canonical_parent(self) -> RackDevice:
            """Get the canonical parent of the View."""
            ...

        @property
        def drum_pads_scroll_position(self) -> int:
            """Access to the index of the lowest visible row of pads. Throws an exception if can_have_drum_pads is false."""
            ...

        @drum_pads_scroll_position.setter
        def drum_pads_scroll_position(self, value: int) -> None: ...

        def drum_pads_scroll_position_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "drum_pads_scroll_position".
            """
            ...

        @property
        def is_collapsed(self) -> bool:
            """Get/Set/Listen if the device is shown collapsed in the device chain."""
            ...

        @is_collapsed.setter
        def is_collapsed(self, value: bool) -> None: ...

        @property
        def is_showing_chain_devices(self) -> bool:
            """Return whether the devices in the currently selected chain are visible. Throws an exception if can_have_chains is false."""
            ...

        @is_showing_chain_devices.setter
        def is_showing_chain_devices(self, value: bool) -> None: ...

        def is_showing_chain_devices_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "is_showing_chain_devices".
            """
            ...

        def remove_drum_pads_scroll_position_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "drum_pads_scroll_position".
            """
            ...

        def remove_is_showing_chain_devices_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "is_showing_chain_devices".
            """
            ...

        def remove_selected_chain_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "selected_chain".
            """
            ...

        def remove_selected_drum_pad_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "selected_drum_pad".
            """
            ...

        @property
        def selected_chain(self) -> Chain | None:
            """Return access to the currently selected chain."""
            ...

        @selected_chain.setter
        def selected_chain(self, value: Chain | None) -> None: ...

        def selected_chain_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "selected_chain".
            """
            ...

        @property
        def selected_drum_pad(self) -> DrumPad:
            """Return access to the currently selected drum pad. Throws an exception if can_have_drum_pads is false."""
            ...

        @selected_drum_pad.setter
        def selected_drum_pad(self, value: DrumPad) -> None: ...

        def selected_drum_pad_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "selected_drum_pad".
            """
            ...

    @property
    def _live_ptr(self) -> int:
        ...

    def add_chains_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "chains" has changed.
        """
        ...

    def add_drum_pads_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "drum_pads" has changed.
        """
        ...

    def add_has_drum_pads_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "has_drum_pads" has changed.
        """
        ...

    def add_has_macro_mappings_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "has_macro_mappings" has changed.
        """
        ...

    def add_is_showing_chains_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "is_showing_chains" has changed.
        """
        ...

    def add_macro(self) -> None:
        """Increases the number of visible macro controls in the rack. Throws an exception if the maximum number of macro controls is reached."""
        ...

    def add_macros_mapped_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "macros_mapped" has changed.
        """
        ...

    def add_return_chains_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "return_chains" has changed.
        """
        ...

    def add_variation_count_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "variation_count" has changed.
        """
        ...

    def add_visible_drum_pads_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "visible_drum_pads" has changed.
        """
        ...

    def add_visible_macro_count_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "visible_macro_count" has changed.
        """
        ...

    @property
    def can_compare_ab(self) -> bool:
        """Returns true if the Device has the capability to AB compare."""
        ...

    @property
    def can_have_chains(self) -> bool:
        """Returns true if the device is a rack."""
        ...

    @property
    def can_have_drum_pads(self) -> bool:
        """Returns true if the device is a drum rack."""
        ...

    @property
    def can_show_chains(self) -> bool:
        """return True, if this Rack contains a rack instrument device that is capable of showing its chains in session view."""
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the Device."""
        ...

    @property
    def chain_selector(self) -> DeviceParameter:
        """Const access to the chain selector parameter."""
        ...

    @property
    def chains(self) -> Vector:
        """Return const access to the list of chains in this device. Throws an exception if can_have_chains is false."""
        ...

    def chains_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "chains".
        """
        ...

    @property
    def class_display_name(self) -> str:
        """Return const access to the name of the device's class name as displayed in Live's browser and device chain"""
        ...

    @property
    def class_name(self) -> str:
        """Return const access to the name of the device's class."""
        ...

    def copy_pad(self, source_index: int | None, destination_index: int | None, /) -> None:
        """Copies all contents of a drum pad from a source pad into a destination pad. copy_pad(source_index, destination_index) where source_index and destination_index correspond to the note number/index of the drum pad in a drum rack. Throws an exception when the source pad is empty, or when the source or destination indices are not between 0 - 127."""
        ...

    def delete_selected_variation(self) -> None:
        """Deletes the currently selected macro variation.Does nothing if there is no selected variation."""
        ...

    @property
    def drum_pads(self) -> Vector[DrumPad]:
        """Return const access to the list of drum pads in this device. Throws an exception if can_have_drum_pads is false."""
        ...

    def drum_pads_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "drum_pads".
        """
        ...

    @property
    def has_drum_pads(self) -> bool:
        """Returns true if the device is a drum rack which has drum pads. Throws an exception if can_have_drum_pads is false."""
        ...

    def has_drum_pads_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "has_drum_pads".
        """
        ...

    @property
    def has_macro_mappings(self) -> bool:
        """Returns true if any of the rack's macros are mapped to a parameter."""
        ...

    def has_macro_mappings_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "has_macro_mappings".
        """
        ...

    def insert_chain(self, index: int = -1, /) -> Chain:
        """Inserts a new chain, either at the specified index or, if not index was specified, at the end of the chain sequence."""
        ...

    @property
    def is_active(self) -> bool:
        """Return const access to whether this device is active. This will be false bothwhen the device is off and when it's inside a rack device which is off."""
        ...

    @property
    def is_showing_chains(self) -> bool:
        """Returns True, if it is showing chains."""
        ...

    @is_showing_chains.setter
    def is_showing_chains(self, value: bool) -> None: ...

    def is_showing_chains_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "is_showing_chains".
        """
        ...

    @property
    def is_using_compare_preset_b(self) -> bool:
        """Returns whether the Device has loaded the preset in compare slot B. Only relevant if can_compare_ab, otherwise errors."""
        ...

    @is_using_compare_preset_b.setter
    def is_using_compare_preset_b(self, value: bool) -> None: ...

    @property
    def latency_in_ms(self) -> float:
        """Returns the latency of the device in ms."""
        ...

    @property
    def latency_in_samples(self) -> int:
        """Returns the latency of the device in samples."""
        ...

    @property
    def macros_mapped(self) -> tuple[bool, ...]:
        """A list of booleans, one for each macro parameter, which is True iffthat macro is mapped to something"""
        ...

    def macros_mapped_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "macros_mapped".
        """
        ...

    @property
    def name(self) -> str:
        """Return access to the name of the device."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    @property
    def parameters(self) -> ATimeableValueVector:
        """Const access to the list of available automatable parameters for this device."""
        ...

    def randomize_macros(self) -> None:
        """Randomizes the values for all macro controls not excluded from randomization."""
        ...

    def recall_last_used_variation(self) -> None:
        """Recalls the macro variation that was recalled most recently.Does nothing if no variation has been recalled yet."""
        ...

    def recall_selected_variation(self) -> None:
        """Recalls the currently selected macro variation.Does nothing if there are no variations."""
        ...

    def remove_chains_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "chains".
        """
        ...

    def remove_drum_pads_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "drum_pads".
        """
        ...

    def remove_has_drum_pads_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "has_drum_pads".
        """
        ...

    def remove_has_macro_mappings_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "has_macro_mappings".
        """
        ...

    def remove_is_showing_chains_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "is_showing_chains".
        """
        ...

    def remove_macro(self) -> None:
        """Decreases the number of visible macro controls in the rack. Throws an exception if the minimum number of macro controls is reached."""
        ...

    def remove_macros_mapped_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "macros_mapped".
        """
        ...

    def remove_return_chains_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "return_chains".
        """
        ...

    def remove_variation_count_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "variation_count".
        """
        ...

    def remove_visible_drum_pads_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "visible_drum_pads".
        """
        ...

    def remove_visible_macro_count_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "visible_macro_count".
        """
        ...

    @property
    def return_chains(self) -> Vector:
        """Return const access to the list of return chains in this device. Throws an exception if can_have_chains is false."""
        ...

    def return_chains_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "return_chains".
        """
        ...

    @property
    def selected_variation_index(self) -> int:
        """Access to the index of the currently selected macro variation.Throws an exception if the index is out of range."""
        ...

    @selected_variation_index.setter
    def selected_variation_index(self, value: int) -> None: ...

    def store_variation(self) -> None:
        """Stores a new variation of the values of all currently mapped macros"""
        ...

    @property
    def type(self) -> DeviceType:
        """Return the type of the device."""
        ...

    @property
    def variation_count(self) -> int:
        """Access to the number of macro variations currently stored."""
        ...

    def variation_count_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "variation_count".
        """
        ...

    @property
    def view(self) -> View:
        """Representing the view aspects of a device."""
        ...

    @property
    def visible_drum_pads(self) -> Vector[DrumPad]:
        """Return const access to the list of visible drum pads in this device. Throws an exception if can_have_drum_pads is false."""
        ...

    def visible_drum_pads_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "visible_drum_pads".
        """
        ...

    @property
    def visible_macro_count(self) -> int:
        """Access to the number of macros that are currently visible."""
        ...

    def visible_macro_count_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "visible_macro_count".
        """
        ...

__all__ = ['RackDevice']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Clip import Clip
    from Live.DrumPad import DrumPad
    from Live.SimplerDevice import SimplerDevice
    from Live.Song import Song
    from Live.Track import Track



class AudioToMidiType(int):
    harmony_to_midi: int = 0
    melody_to_midi: int = 1
    drums_to_midi: int = 2

def audio_to_midi_clip(song: Song | None, audio_clip: Clip | None, audio_to_midi_type: int | None, /) -> None:
    """
    Creates a MIDI clip in a new MIDI track with the notes extracted from the given
    audio_clip. The `audio_to_midi_type` decides which algorithm is used in
    the process. Raises error when called with an inconvertible clip or invalid
    `audio_to_midi_type`.
    """
    ...

def create_drum_rack_from_audio_clip(song: Song | None, audio_clip: Clip | None, /) -> None:
    """
    Creates a new track with a drum rack with a simpler on the first pad with
    the specified audio clip.
    """
    ...

def create_midi_track_from_drum_pad(song: Song | None, drum_pad: DrumPad | None, /) -> None:
    """Creates a new Midi track containing the specified Drum Pad's device chain."""
    ...

def create_midi_track_with_simpler(song: Song | None, audio_clip: Clip | None, /) -> None:
    """Creates a new Midi track with a simpler including the specified audio clip."""
    ...

def is_convertible_to_midi(song: Song | None, audio_clip: Clip | None, /) -> bool:
    """
    Returns whether `audio_clip` can be converted to MIDI.
    Raises error when called with a MIDI clip
    """
    ...

def move_devices_on_track_to_new_drum_rack_pad(song: Song | None, track_index: int | None, /) -> Track:
    """
    Moves the entire device chain of the track according to the track index
    onto the C1 (note 36) drum pad of a new drum rack in a new track.If the track associated with the track index does not contain any devices
    nothing changes (i.e. a new track and new drum rack are not created).
    """
    ...

def sliced_simpler_to_drum_rack(song: Song | None, simpler: SimplerDevice | None, /) -> None:
    """
    Converts the Simpler into a Drum Rack, assigning each slice to a drum pad.
    Calling it on a non-sliced simpler raises an error.
    """
    ...

__all__ = ['AudioToMidiType', 'audio_to_midi_clip', 'create_drum_rack_from_audio_clip', 'create_midi_track_from_drum_pad', 'create_midi_track_with_simpler', 'is_convertible_to_midi', 'move_devices_on_track_to_new_drum_rack_pad', 'sliced_simpler_to_drum_rack']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Device import ATimeableValueVector, Device, DeviceType
    from Live.Track import Track



class Eq8Device(Device):
    """This class represents an Eq8 device."""

    class View(Device.View):
        """Representing the view aspects of an Eq8 device."""

        @property
        def _live_ptr(self) -> int:
            ...

        def add_selected_band_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "selected_band" has changed.
            """
            ...

        @property
        def canonical_parent(self) -> Eq8Device:
            """Get the canonical parent of the View."""
            ...

        @property
        def is_collapsed(self) -> bool:
            """Get/Set/Listen if the device is shown collapsed in the device chain."""
            ...

        @is_collapsed.setter
        def is_collapsed(self, value: bool) -> None: ...

        def remove_selected_band_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "selected_band".
            """
            ...

        @property
        def selected_band(self) -> int:
            """Access to the selected filter band."""
            ...

        @selected_band.setter
        def selected_band(self, value: int) -> None: ...

        def selected_band_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "selected_band".
            """
            ...

    @property
    def _live_ptr(self) -> int:
        ...

    def add_edit_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "edit_mode" has changed.
        """
        ...

    def add_global_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "global_mode" has changed.
        """
        ...

    def add_oversample_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "oversample" has changed.
        """
        ...

    @property
    def can_compare_ab(self) -> bool:
        """Returns true if the Device has the capability to AB compare."""
        ...

    @property
    def can_have_chains(self) -> bool:
        """Returns true if the device is a rack."""
        ...

    @property
    def can_have_drum_pads(self) -> bool:
        """Returns true if the device is a drum rack."""
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the Device."""
        ...

    @property
    def class_display_name(self) -> str:
        """Return const access to the name of the device's class name as displayed in Live's browser and device chain"""
        ...

    @property
    def class_name(self) -> str:
        """Return const access to the name of the device's class."""
        ...

    @property
    def edit_mode(self) -> bool:
        """Access to Eq8's edit mode."""
        ...

    @edit_mode.setter
    def edit_mode(self, value: bool) -> None: ...

    def edit_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "edit_mode".
        """
        ...

    @property
    def global_mode(self) -> int:
        """Access to Eq8's global mode."""
        ...

    @global_mode.setter
    def global_mode(self, value: int) -> None: ...

    def global_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "global_mode".
        """
        ...

    @property
    def is_active(self) -> bool:
        """Return const access to whether this device is active. This will be false bothwhen the device is off and when it's inside a rack device which is off."""
        ...

    @property
    def is_using_compare_preset_b(self) -> bool:
        """Returns whether the Device has loaded the preset in compare slot B. Only relevant if can_compare_ab, otherwise errors."""
        ...

    @is_using_compare_preset_b.setter
    def is_using_compare_preset_b(self, value: bool) -> None: ...

    @property
    def latency_in_ms(self) -> float:
        """Returns the latency of the device in ms."""
        ...

    @property
    def latency_in_samples(self) -> int:
        """Returns the latency of the device in samples."""
        ...

    @property
    def name(self) -> str:
        """Return access to the name of the device."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    @property
    def oversample(self) -> bool:
        """Access to Eq8's oversample value."""
        ...

    @oversample.setter
    def oversample(self, value: bool) -> None: ...

    def oversample_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "oversample".
        """
        ...

    @property
    def parameters(self) -> ATimeableValueVector:
        """Const access to the list of available automatable parameters for this device."""
        ...

    def remove_edit_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "edit_mode".
        """
        ...

    def remove_global_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "global_mode".
        """
        ...

    def remove_oversample_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "oversample".
        """
        ...

    @property
    def type(self) -> DeviceType:
        """Return the type of the device."""
        ...

    @property
    def view(self) -> View:
        """Representing the view aspects of a device."""
        ...

class EditMode(int):
    a: int = 0
    b: int = 1

class GlobalMode(int):
    stereo: int = 0
    left_right: int = 1
    mid_side: int = 2

__all__ = ['Eq8Device', 'EditMode', 'GlobalMode']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import IntVector
    from Live.Device import ATimeableValueVector, Device, DeviceType
    from Live.Sample import Sample
    from Live.Track import Track



class SimplerDevice(Device):
    """This class represents a Simpler device."""

    class View(Device.View):
        """Representing the view aspects of a simpler device."""

        @property
        def _live_ptr(self) -> int:
            ...

        def add_sample_end_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "sample_end" has changed.
            """
            ...

        def add_sample_env_fade_in_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "sample_env_fade_in" has changed.
            """
            ...

        def add_sample_env_fade_out_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "sample_env_fade_out" has changed.
            """
            ...

        def add_sample_loop_end_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "sample_loop_end" has changed.
            """
            ...

        def add_sample_loop_fade_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "sample_loop_fade" has changed.
            """
            ...

        def add_sample_loop_start_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "sample_loop_start" has changed.
            """
            ...

        def add_sample_start_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "sample_start" has changed.
            """
            ...

        def add_selected_slice_listener(self, callback: Callable | None, /) -> None:
            """
            Add a listener function or method, which will be called as soon as the
            property "selected_slice" has changed.
            """
            ...

        @property
        def canonical_parent(self) -> SimplerDevice:
            """Get the canonical parent of the View."""
            ...

        @property
        def is_collapsed(self) -> bool:
            """Get/Set/Listen if the device is shown collapsed in the device chain."""
            ...

        @is_collapsed.setter
        def is_collapsed(self, value: bool) -> None: ...

        def remove_sample_end_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "sample_end".
            """
            ...

        def remove_sample_env_fade_in_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "sample_env_fade_in".
            """
            ...

        def remove_sample_env_fade_out_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "sample_env_fade_out".
            """
            ...

        def remove_sample_loop_end_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "sample_loop_end".
            """
            ...

        def remove_sample_loop_fade_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "sample_loop_fade".
            """
            ...

        def remove_sample_loop_start_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "sample_loop_start".
            """
            ...

        def remove_sample_start_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "sample_start".
            """
            ...

        def remove_selected_slice_listener(self, callback: Callable | None, /) -> None:
            """
            Remove a previously set listener function or method from
            property "selected_slice".
            """
            ...

        @property
        def sample_end(self) -> int:
            """Access to the modulated samples end position in samples. Returns -1 in case there is no sample loaded."""
            ...

        def sample_end_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "sample_end".
            """
            ...

        @property
        def sample_env_fade_in(self) -> int:
            """Access to the envelope fade-in time in samples. Returned value is only in use when Simpler is in one-shot mode. Returns -1 in case there is no sample loaded."""
            ...

        def sample_env_fade_in_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "sample_env_fade_in".
            """
            ...

        @property
        def sample_env_fade_out(self) -> int:
            """Access to the envelope fade-out time in samples. Returned value is only in use when Simpler is in one-shot mode. Returns -1 in case there is no sample loaded."""
            ...

        def sample_env_fade_out_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "sample_env_fade_out".
            """
            ...

        @property
        def sample_loop_end(self) -> int:
            """Access to the modulated samples loop end position in samples. Returns -1 in case there is no sample loaded."""
            ...

        def sample_loop_end_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "sample_loop_end".
            """
            ...

        @property
        def sample_loop_fade(self) -> int:
            """Access to the modulated samples loop fade position in samples. Returns -1 in case there is no sample loaded."""
            ...

        def sample_loop_fade_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "sample_loop_fade".
            """
            ...

        @property
        def sample_loop_start(self) -> int:
            """Access to the modulated samples loop start position in samples. Returns -1 in case there is no sample loaded."""
            ...

        def sample_loop_start_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "sample_loop_start".
            """
            ...

        @property
        def sample_start(self) -> int:
            """Access to the modulated samples start position in samples. Returns -1 in case there is no sample loaded."""
            ...

        def sample_start_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "sample_start".
            """
            ...

        @property
        def selected_slice(self) -> int:
            """Access to the selected slice."""
            ...

        @selected_slice.setter
        def selected_slice(self, value: int) -> None: ...

        def selected_slice_has_listener(self, callback: Callable | None, /) -> bool:
            """
            Returns true, if the given listener function or method is connected
            to the property "selected_slice".
            """
            ...

    @property
    def _live_ptr(self) -> int:
        ...

    def add_can_warp_as_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "can_warp_as" has changed.
        """
        ...

    def add_can_warp_double_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "can_warp_double" has changed.
        """
        ...

    def add_can_warp_half_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "can_warp_half" has changed.
        """
        ...

    def add_multi_sample_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "multi_sample_mode" has changed.
        """
        ...

    def add_note_pitch_bend_range_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "note_pitch_bend_range" has changed.
        """
        ...

    def add_pad_slicing_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "pad_slicing" has changed.
        """
        ...

    def add_pitch_bend_range_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "pitch_bend_range" has changed.
        """
        ...

    def add_playback_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "playback_mode" has changed.
        """
        ...

    def add_playing_position_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "playing_position_enabled" has changed.
        """
        ...

    def add_playing_position_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "playing_position" has changed.
        """
        ...

    def add_retrigger_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "retrigger" has changed.
        """
        ...

    def add_sample_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "sample" has changed.
        """
        ...

    def add_slicing_playback_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "slicing_playback_mode" has changed.
        """
        ...

    def add_voices_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "voices" has changed.
        """
        ...

    @property
    def can_compare_ab(self) -> bool:
        """Returns true if the Device has the capability to AB compare."""
        ...

    @property
    def can_have_chains(self) -> bool:
        """Returns true if the device is a rack."""
        ...

    @property
    def can_have_drum_pads(self) -> bool:
        """Returns true if the device is a drum rack."""
        ...

    @property
    def can_warp_as(self) -> bool:
        """Returns true if warp_as is available."""
        ...

    def can_warp_as_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "can_warp_as".
        """
        ...

    @property
    def can_warp_double(self) -> bool:
        """Returns true if warp_double is available."""
        ...

    def can_warp_double_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "can_warp_double".
        """
        ...

    @property
    def can_warp_half(self) -> bool:
        """Returns true if warp_half is available."""
        ...

    def can_warp_half_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "can_warp_half".
        """
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the Device."""
        ...

    @property
    def class_display_name(self) -> str:
        """Return const access to the name of the device's class name as displayed in Live's browser and device chain"""
        ...

    @property
    def class_name(self) -> str:
        """Return const access to the name of the device's class."""
        ...

    def crop(self) -> None:
        """
        Crop the loaded sample to the active area between start- and end marker.
        Calling this method on an empty simpler raises an error.
        """
        ...

    def guess_playback_length(self) -> float:
        """
        Return an estimated beat time for the playback length between start- and end-marker.
        Calling this method on an empty simpler raises an error.
        """
        ...

    @property
    def is_active(self) -> bool:
        """Return const access to whether this device is active. This will be false bothwhen the device is off and when it's inside a rack device which is off."""
        ...

    @property
    def is_using_compare_preset_b(self) -> bool:
        """Returns whether the Device has loaded the preset in compare slot B. Only relevant if can_compare_ab, otherwise errors."""
        ...

    @is_using_compare_preset_b.setter
    def is_using_compare_preset_b(self, value: bool) -> None: ...

    @property
    def latency_in_ms(self) -> float:
        """Returns the latency of the device in ms."""
        ...

    @property
    def latency_in_samples(self) -> int:
        """Returns the latency of the device in samples."""
        ...

    @property
    def multi_sample_mode(self) -> bool:
        """Returns whether Simpler is in mulit-sample mode."""
        ...

    def multi_sample_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "multi_sample_mode".
        """
        ...

    @property
    def name(self) -> str:
        """Return access to the name of the device."""
        ...

    @name.setter
    def name(self, value: str) -> None: ...

    @property
    def note_pitch_bend_range(self) -> int:
        """Access to the Note Pitch Bend Range in Simpler."""
        ...

    @note_pitch_bend_range.setter
    def note_pitch_bend_range(self, value: int) -> None: ...

    def note_pitch_bend_range_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "note_pitch_bend_range".
        """
        ...

    @property
    def pad_slicing(self) -> bool:
        """
        When set to true, slices can be added in slicing mode by playing notes
        .that are not assigned to slices, yet.
        """
        ...

    @pad_slicing.setter
    def pad_slicing(self, value: bool) -> None: ...

    def pad_slicing_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "pad_slicing".
        """
        ...

    @property
    def parameters(self) -> ATimeableValueVector:
        """Const access to the list of available automatable parameters for this device."""
        ...

    @property
    def pitch_bend_range(self) -> int:
        """Access to the Pitch Bend Range in Simpler."""
        ...

    @pitch_bend_range.setter
    def pitch_bend_range(self, value: int) -> None: ...

    def pitch_bend_range_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "pitch_bend_range".
        """
        ...

    @property
    def playback_mode(self) -> int:
        """Access to Simpler's playback mode."""
        ...

    @playback_mode.setter
    def playback_mode(self, value: int) -> None: ...

    def playback_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "playback_mode".
        """
        ...

    @property
    def playing_position(self) -> float:
        """
        Constant access to the current playing position in the sample.
        The returned value is the normalized position between sample start and end.
        """
        ...

    @property
    def playing_position_enabled(self) -> bool:
        """
        Returns whether Simpler is showing the playing position.
        The returned value is True while the sample is played back
        """
        ...

    def playing_position_enabled_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "playing_position_enabled".
        """
        ...

    def playing_position_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "playing_position".
        """
        ...

    def remove_can_warp_as_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "can_warp_as".
        """
        ...

    def remove_can_warp_double_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "can_warp_double".
        """
        ...

    def remove_can_warp_half_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "can_warp_half".
        """
        ...

    def remove_multi_sample_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "multi_sample_mode".
        """
        ...

    def remove_note_pitch_bend_range_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "note_pitch_bend_range".
        """
        ...

    def remove_pad_slicing_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "pad_slicing".
        """
        ...

    def remove_pitch_bend_range_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "pitch_bend_range".
        """
        ...

    def remove_playback_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "playback_mode".
        """
        ...

    def remove_playing_position_enabled_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "playing_position_enabled".
        """
        ...

    def remove_playing_position_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "playing_position".
        """
        ...

    def remove_retrigger_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "retrigger".
        """
        ...

    def remove_sample_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "sample".
        """
        ...

    def remove_slicing_playback_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "slicing_playback_mode".
        """
        ...

    def remove_voices_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "voices".
        """
        ...

    @property
    def retrigger(self) -> bool:
        """Access to Simpler's retrigger mode."""
        ...

    @retrigger.setter
    def retrigger(self, value: bool) -> None: ...

    def retrigger_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "retrigger".
        """
        ...

    def reverse(self) -> None:
        """
        Reverse the loaded sample.
        Calling this method on an empty simpler raises an error.
        """
        ...

    @property
    def sample(self) -> Sample:
        """Get the loaded Sample."""
        ...

    def sample_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "sample".
        """
        ...

    @property
    def slicing_playback_mode(self) -> int:
        """Access to Simpler's slicing playback mode."""
        ...

    @slicing_playback_mode.setter
    def slicing_playback_mode(self, value: int) -> None: ...

    def slicing_playback_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "slicing_playback_mode".
        """
        ...

    @property
    def type(self) -> DeviceType:
        """Return the type of the device."""
        ...

    @property
    def view(self) -> View:
        """Representing the view aspects of a device."""
        ...

    @property
    def voices(self) -> int:
        """Access to the number of voices in Simpler."""
        ...

    @voices.setter
    def voices(self, value: int) -> None: ...

    def voices_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "voices".
        """
        ...

    def warp_as(self, beat_time: float | None, /) -> None:
        """
        Warp the playback region between start- and end-marker as the given length.
        Calling this method on an empty simpler raises an error.
        """
        ...

    def warp_double(self) -> None:
        """Doubles the tempo for region between start- and end-marker."""
        ...

    def warp_half(self) -> None:
        """Halves the tempo for region between start- and end-marker."""
        ...

class PlaybackMode(int):
    classic: int = 0
    one_shot: int = 1
    slicing: int = 2

class SlicingPlaybackMode(int):
    mono: int = 0
    poly: int = 1
    thru: int = 2

def get_available_voice_numbers() -> IntVector:
    """Get a vector of valid Simpler voice numbers."""
    ...

__all__ = ['SimplerDevice', 'PlaybackMode', 'SlicingPlaybackMode', 'get_available_voice_numbers']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable

if TYPE_CHECKING:
    from Live.Base import Vector
    from Live.Device import Device
    from Live.DeviceParameter import DeviceParameter
    from Live.LomObject import LomObject
    from Live.Song import Song
    from Live.Track import Track



class MixerDevice(LomObject):
    """
    This class represents a Mixer Device in Live, which gives you
    access to the Volume and Panning properties of a Track.
    """

    @property
    def _live_ptr(self) -> int:
        ...

    def add_crossfade_assign_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "crossfade_assign" has changed.
        """
        ...

    def add_panning_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "panning_mode" has changed.
        """
        ...

    def add_sends_listener(self, callback: Callable | None, /) -> None:
        """
        Add a listener function or method, which will be called as soon as the
        property "sends" has changed.
        """
        ...

    @property
    def canonical_parent(self) -> Track:
        """Get the canonical parent of the mixer device."""
        ...

    @property
    def crossfade_assign(self) -> int:
        """Player- and ReturnTracks only: Access to the Track's Crossfade Assign State."""
        ...

    @crossfade_assign.setter
    def crossfade_assign(self, value: int) -> None: ...

    def crossfade_assign_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "crossfade_assign".
        """
        ...

    class crossfade_assignments(int):
        A: int = 0
        NONE: int = 1
        B: int = 2

    @property
    def crossfader(self) -> DeviceParameter:
        """MainTrack only: Const access to the Crossfader."""
        ...

    @property
    def cue_volume(self) -> DeviceParameter:
        """MainTrack only: Const access to the Cue Volume Parameter."""
        ...

    @property
    def left_split_stereo(self) -> DeviceParameter:
        """Const access to the Track's Left Split Stereo Panning Device Parameter."""
        ...

    @property
    def panning(self) -> DeviceParameter:
        """Const access to the Tracks Panning Device Parameter."""
        ...

    @property
    def panning_mode(self) -> int:
        """Access to the Track's Panning Mode."""
        ...

    @panning_mode.setter
    def panning_mode(self, value: int) -> None: ...

    def panning_mode_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "panning_mode".
        """
        ...

    class panning_modes(int):
        stereo: int = 0
        stereo_split: int = 1

    def remove_crossfade_assign_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "crossfade_assign".
        """
        ...

    def remove_panning_mode_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "panning_mode".
        """
        ...

    def remove_sends_listener(self, callback: Callable | None, /) -> None:
        """
        Remove a previously set listener function or method from
        property "sends".
        """
        ...

    @property
    def right_split_stereo(self) -> DeviceParameter:
        """Const access to the Track's Right Split Stereo Panning Device Parameter."""
        ...

    @property
    def sends(self) -> Vector[DeviceParameter]:
        """Const access to the Tracks list of Send Amount Device Parameters."""
        ...

    def sends_has_listener(self, callback: Callable | None, /) -> bool:
        """
        Returns true, if the given listener function or method is connected
        to the property "sends".
        """
        ...

    @property
    def song_tempo(self) -> DeviceParameter:
        """MainTrack only: Const access to the Song's Tempo."""
        ...

    @property
    def track_activator(self) -> DeviceParameter:
        """Const access to the Tracks Activator Device Parameter."""
        ...

    @property
    def volume(self) -> DeviceParameter:
        """Const access to the Tracks Volume Device Parameter."""
        ...

__all__ = ['MixerDevice']



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Callable, Iterable


class LomObject:
    """this is the base class for an object that is accessible via the LOM"""

    @property
    def _live_ptr(self) -> int:
        ...

__all__ = ['LomObject']
